﻿using Microsoft.AspNetCore.Mvc;
using System.IO;
using Microsoft.AspNetCore.Hosting;
using System.Net.Http;
using System;
using Microsoft.AspNetCore.Http;
using System.Collections.Generic;
using Syncfusion.DocIO.DLS;
using Syncfusion.DocIO;

namespace DocIO_AspNetCore_WebApplication.Controllers
{
    public class HomeController : Controller
    {
        private readonly IHostingEnvironment _hostingEnvironment;
        public HomeController(IHostingEnvironment hostingEnvironment)
        {
            _hostingEnvironment = hostingEnvironment;
        }
        private MemoryStream outputStream = null;

        #region Add and Retrieve Image caption
        public async System.Threading.Tasks.Task<ActionResult> Index(string button)
        {
            
            if (button == null)
                return View();

            // Gets file path for input.
            string rootPath = _hostingEnvironment.WebRootPath;

            WordDocument document = new WordDocument();
            document.EnsureMinimal();
            document.LastParagraph.AppendText("First paragraph");

            IWParagraph paragraph = document.LastSection.AddParagraph();
            //Add bookmark with the picture.
            paragraph.AppendBookmarkStart("Image");

            FileStream fileStream = new FileStream(rootPath + "\\DocIO\\Image.jpg", FileMode.Open);            
            IWPicture picture = paragraph.AppendPicture(fileStream);
            paragraph.AppendBookmarkEnd("Image");

            //Add caption for the picture.
            AddCaption(picture, "Image caption", CaptionNumberingFormat.Number, CaptionPosition.AboveImage);

            //Navigate to the image, using its bookmark.
            BookmarksNavigator bookmarkNavigator = new BookmarksNavigator(document);
            bookmarkNavigator.MoveToBookmark("Image");
            TextBodyPart part = bookmarkNavigator.GetBookmarkContent();

            //Retrieves the caption text from of the image.
            string pictureCaption = RetriveImageCaption(part);


            #region Saving document
            MemoryStream memoryStream = new MemoryStream();
            //Save the document into memory stream
            document.Save(memoryStream, FormatType.Docx);
            //Closes the Word document instance
            document.Close();
            memoryStream.Position = 0;
            return File(memoryStream, "application/msword", "Sample.docx");
            #endregion
        }
      

        #region Adding caption

        /// <summary>
        /// Adds caption to the picture and maintain with bookmark.
        /// </summary>
        /// <param name="picture">Picture to add caption</param>
        /// <param name="captionText">Caption name to add.</param>
        /// <param name="captionNumberingFormat">CaptionNumberingFormat to set numbering format.</param>
        /// <param name="captionPosition">CaptionPosition to set the position of caption.</param>
        public static void AddCaption(IWPicture picture, string captionText, CaptionNumberingFormat captionNumberingFormat, CaptionPosition captionPosition)
        {
            WParagraph paragraph;

            BookmarkStart bookmarkstart = picture.PreviousSibling is BookmarkStart ? picture.PreviousSibling as BookmarkStart : null;
            BookmarkEnd bookmarkEnd = picture.NextSibling is BookmarkEnd ? picture.NextSibling as BookmarkEnd : null;

            //Adds caption as usual.
            picture.AddCaption(captionText, captionNumberingFormat, captionPosition);

            //For the caption after the image.
            if (captionPosition == CaptionPosition.AfterImage)
            {
                //Gets the paragraph contains caption.
                paragraph = picture.OwnerParagraph.NextSibling is WParagraph ? picture.OwnerParagraph.NextSibling as WParagraph : null;
                //Moves the bookmark end to the end of caption.
                paragraph.ChildEntities.Add(bookmarkEnd);
            }
            //For the caption above the image.
            else
            {
                //Get the paragraph contains caption.
                paragraph = picture.OwnerParagraph.PreviousSibling is WParagraph ? picture.OwnerParagraph.PreviousSibling as WParagraph : null;
                //Moves the bookmark start to the start of caption.
                paragraph.ChildEntities.Insert(0, bookmarkstart);
                //Moves the bookmark end to the end of picture.
                picture.OwnerParagraph.ChildEntities.Add(bookmarkEnd);
            }
        }
        #endregion

        #region Retrieving caption text.
        /// <summary>
        /// Retrieves the caption of the image.
        /// </summary>
        /// <param name="part">Represents a bookmark content contains picture and its caption.</param>
        /// <returns></returns>
        private static string RetriveImageCaption(TextBodyPart part)
        {
            string captionText = "";
            //Iterate to body items and get the caption text.
            foreach (IEntity entity in part.BodyItems)
            {
                switch (entity.EntityType)
                {
                    case EntityType.Paragraph:
                        WParagraph paragraph = entity as WParagraph;

                        for (int i = 0; i < paragraph.ChildEntities.Count; i++)
                        {
                            if (paragraph.ChildEntities[i] is WTextRange)
                                captionText += (paragraph.ChildEntities[i] as WTextRange).Text;
                        }
                        break;
                    default:
                        break;
                }
            }
            return captionText;
        }
        #endregion
        #endregion
        public IActionResult About()
        {
            ViewData["Message"] = "Your application description page.";

            return View();
        }

        public IActionResult Contact()
        {
            ViewData["Message"] = "Your contact page.";

            return View();
        }

        public IActionResult Error()
        {
            return View();
        }
    }
}